@@ -45,7 +45,7 @@ impl std::fmt::Debug for FP12 {
     fn fmt(&self, formatter: &mut std::fmt::Formatter) -> std::fmt::Result {
         write!(formatter, "{}", self.tostring())
     }
-}
+}    
 
 #[cfg(feature = "std")]
 impl std::fmt::Display for FP12 {
@@ -138,7 +138,7 @@ impl FP12 {
         self.b.cmove(&g.b, d);
         self.c.cmove(&g.c, d);
         let mut u = d as usize;
-        u = !(u - 1);
+        u = !(u.wrapping_sub(1));
         self.stype ^= (self.stype ^ g.stype) & u;
     }
 
@@ -870,44 +870,44 @@ impl FP12 {
 
     /* convert from byte array to FP12 */
     pub fn frombytes(w: &[u8]) -> FP12 {
-        const MB: usize = 4 * (big::MODBYTES as usize);
+        const MB:usize = 4*(big::MODBYTES as usize);
         let mut t: [u8; MB] = [0; MB];
-        for i in 0..MB {
-            t[i] = w[i];
-        }
-        let c = FP4::frombytes(&t);
-        for i in 0..MB {
-            t[i] = w[i + MB];
-        }
-        let b = FP4::frombytes(&t);
-        for i in 0..MB {
-            t[i] = w[i + 2 * MB];
-        }
-        let a = FP4::frombytes(&t);
-        FP12::new_fp4s(&a, &b, &c)
+	    for i in 0..MB {
+		    t[i]=w[i];
+	    }
+        let c=FP4::frombytes(&t);
+	    for i in 0..MB {
+		    t[i]=w[i+MB];
+	    }
+        let b=FP4::frombytes(&t);
+	    for i in 0..MB {
+		    t[i]=w[i+2*MB];
+	    }
+        let a=FP4::frombytes(&t);
+	    FP12::new_fp4s(&a,&b,&c)
     }
 
     /* convert this to byte array */
     pub fn tobytes(&mut self, w: &mut [u8]) {
-        const MB: usize = 4 * (big::MODBYTES as usize);
+        const MB:usize = 4*(big::MODBYTES as usize);
         let mut t: [u8; MB] = [0; MB];
 
         self.c.tobytes(&mut t);
-        for i in 0..MB {
-            w[i] = t[i];
-        }
+	    for i in 0..MB { 
+		    w[i]=t[i];
+	    }
         self.b.tobytes(&mut t);
-        for i in 0..MB {
-            w[i + MB] = t[i];
-        }
+	    for i in 0..MB {
+		    w[i+MB]=t[i];
+	    }
         self.a.tobytes(&mut t);
-        for i in 0..MB {
-            w[i + 2 * MB] = t[i];
-        }
+	    for i in 0..MB {
+		    w[i+2*MB]=t[i];
+	    }
     }
 
     /* output to hex string */
-    #[cfg(feature = "std")]
+#[cfg(feature = "std")]
     pub fn tostring(&self) -> String {
         format!(
             "[{},{},{}]",
@@ -917,9 +917,9 @@ impl FP12 {
         )
     }
 
-    /* Note this is simple square and multiply, so not side-channel safe */
-    /* But fast for final exponentiation where exponent is not a secret */
-    /* return this^e */
+/* Note this is simple square and multiply, so not side-channel safe */
+/* But fast for final exponentiation where exponent is not a secret */
+/* return this^e */
     pub fn pow(&self, e: &BIG) -> FP12 {
         let mut r = FP12::new_copy(self);
         r.norm();
