@@ -621,7 +621,8 @@ impl ECP4 {
         }
         w[nb] = (t.lastbits(5)) as i8;
 
-        P.copy(&W[((w[nb] as usize) - 1) / 2]);
+        //P.copy(&W[((w[nb] as usize) - 1) / 2]);
+        P.selector(&W, w[nb] as i32);
         for i in (0..nb).rev() {
             Q.selector(&W, w[i] as i32);
             P.dbl();
