@@ -611,16 +611,19 @@ pub fn fexp(m: &FP24) -> FP24 {
 
 #[allow(non_snake_case)]
 /* GLV method */
-fn glv(e: &BIG) -> [BIG; 2] {
+fn glv(ee: &BIG) -> [BIG; 2] {
     let mut u: [BIG; 2] = [BIG::new(), BIG::new()];
     let q = BIG::new_ints(&rom::CURVE_ORDER);
     let mut x = BIG::new_ints(&rom::CURVE_BNX);
     let x2 = BIG::smul(&x, &x);
+
     x.copy(&BIG::smul(&x2, &x2));
-    u[0].copy(&e);
-    u[0].rmod(&x);
-    u[1].copy(&e);
-    u[1].div(&x);
+    let bd=q.nbits()-x.nbits();
+
+    u[0].copy(&ee);
+    u[0].ctmod(&x,bd);
+    u[1].copy(&ee);
+    u[1].ctdiv(&x,bd);
     u[1].rsub(&q);
 
     u
@@ -628,7 +631,7 @@ fn glv(e: &BIG) -> [BIG; 2] {
 
 #[allow(non_snake_case)]
 /* Galbraith & Scott Method */
-pub fn gs(e: &BIG) -> [BIG; 8] {
+pub fn gs(ee: &BIG) -> [BIG; 8] {
     let mut u: [BIG; 8] = [
         BIG::new(),
         BIG::new(),
@@ -641,11 +644,14 @@ pub fn gs(e: &BIG) -> [BIG; 8] {
     ];
     let q = BIG::new_ints(&rom::CURVE_ORDER);
     let x = BIG::new_ints(&rom::CURVE_BNX);
-    let mut w = BIG::new_copy(&e);
+
+    let bd=q.nbits()-x.nbits();  // fixed
+
+    let mut w = BIG::new_copy(&ee);
     for i in 0..7 {
         u[i].copy(&w);
-        u[i].rmod(&x);
-        w.div(&x);
+        u[i].ctmod(&x,bd);
+        w.ctdiv(&x,bd);
     }
     u[7].copy(&w);
     if ecp::SIGN_OF_X == ecp::NEGATIVEX {
@@ -666,14 +672,17 @@ pub fn gs(e: &BIG) -> [BIG; 8] {
 /* Multiply P by e in group G1 */
 pub fn g1mul(P: &ECP, e: &BIG) -> ECP {
     let mut R = ECP::new();
+    let q = BIG::new_ints(&rom::CURVE_ORDER);
+    let mut ee= BIG::new_copy(e);
+    ee.rmod(&q);
     if rom::USE_GLV {
         R.copy(P);
         let mut Q = ECP::new();
         Q.copy(P);
         Q.affine();
-        let q = BIG::new_ints(&rom::CURVE_ORDER);
+
         let mut cru = FP::new_big(&BIG::new_ints(&rom::CRU));
-        let mut u = glv(e);
+        let mut u = glv(&ee);
         Q.mulx(&mut cru);
 
         let mut np = u[0].nbits();
@@ -695,7 +704,7 @@ pub fn g1mul(P: &ECP, e: &BIG) -> ECP {
         u[1].norm();
         R = R.mul2(&u[0], &mut Q, &u[1]);
     } else {
-        R = P.mul(e);
+        R = P.clmul(&ee,&q);
     }
     R
 }
@@ -704,6 +713,9 @@ pub fn g1mul(P: &ECP, e: &BIG) -> ECP {
 /* Multiply P by e in group G2 */
 pub fn g2mul(P: &ECP4, e: &BIG) -> ECP4 {
     let mut R = ECP4::new();
+    let q = BIG::new_ints(&rom::CURVE_ORDER);
+    let mut ee= BIG::new_copy(e);
+    ee.rmod(&q);
     if rom::USE_GS_G2 {
         let mut Q: [ECP4; 8] = [
             ECP4::new(),
@@ -715,8 +727,8 @@ pub fn g2mul(P: &ECP4, e: &BIG) -> ECP4 {
             ECP4::new(),
             ECP4::new(),
         ];
-        let q = BIG::new_ints(&rom::CURVE_ORDER);
-        let mut u = gs(e);
+
+        let mut u = gs(&ee);
         let mut T = ECP4::new();
 
         let f = ECP4::frob_constants();
@@ -741,7 +753,7 @@ pub fn g2mul(P: &ECP4, e: &BIG) -> ECP4 {
 
         R.copy(&ECP4::mul8(&mut Q, &u));
     } else {
-        R.copy(&P.mul(e));
+        R.copy(&P.mul(&ee));
     }
     R
 }
@@ -750,6 +762,9 @@ pub fn g2mul(P: &ECP4, e: &BIG) -> ECP4 {
 /* Note that this method requires a lot of RAM! Better to use compressed XTR method, see FP4.java */
 pub fn gtpow(d: &FP24, e: &BIG) -> FP24 {
     let mut r = FP24::new();
+    let q = BIG::new_ints(&rom::CURVE_ORDER);
+    let mut ee= BIG::new_copy(e);
+    ee.rmod(&q);
     if rom::USE_GS_GT {
         let mut g: [FP24; 8] = [
             FP24::new(),
@@ -762,9 +777,9 @@ pub fn gtpow(d: &FP24, e: &BIG) -> FP24 {
             FP24::new(),
         ];
         let f = FP2::new_bigs(&BIG::new_ints(&rom::FRA), &BIG::new_ints(&rom::FRB));
-        let q = BIG::new_ints(&rom::CURVE_ORDER);
+
         let mut t = BIG::new();
-        let mut u = gs(e);
+        let mut u = gs(&ee);
         let mut w = FP24::new();
 
         g[0].copy(&d);
@@ -785,7 +800,7 @@ pub fn gtpow(d: &FP24, e: &BIG) -> FP24 {
         }
         r.copy(&FP24::pow8(&mut g, &u));
     } else {
-        r.copy(&d.pow(e));
+        r.copy(&d.pow(&ee));
     }
     r
 }
@@ -793,34 +808,62 @@ pub fn gtpow(d: &FP24, e: &BIG) -> FP24 {
 /* test G1 group membership */
 #[allow(non_snake_case)]
 pub fn g1member(P: &ECP) -> bool {
-    let q = BIG::new_ints(&rom::CURVE_ORDER);
+    //let q = BIG::new_ints(&rom::CURVE_ORDER);
     if P.is_infinity() {
         return false;
     }
-    let W=g1mul(&P,&q);
-    if !W.is_infinity() {
+    let x = BIG::new_ints(&rom::CURVE_BNX);
+    let mut cru = FP::new_big(&BIG::new_ints(&rom::CRU));
+    let mut W=ECP::new(); W.copy(P); W.mulx(&mut cru);
+    let mut T=P.mul(&x); 
+    if P.equals(&T) {return false;}    // P is of low order      
+    T=T.mul(&x); T=T.mul(&x); T=T.mul(&x); T.neg();
+    if !W.equals(&T) {
         return false;
     }
+
+// Not needed
+//    W.add(P); T.mulx(&mut cru); W.add(&T);
+//    if !W.is_infinity() {
+//        return false;
+//    }   
+
+/*
+    let W=P.mul(&q); 
+    if !W.is_infinity() {
+        return false;
+    } */
     true
 }
 
 /* test G2 group membership */
 #[allow(non_snake_case)]
 pub fn g2member(P: &ECP4) -> bool {
-    let q = BIG::new_ints(&rom::CURVE_ORDER);
+    //let q = BIG::new_ints(&rom::CURVE_ORDER);
     if P.is_infinity() {
         return false;
     }
-    let W=g2mul(&P,&q);
-    if !W.is_infinity() {
+    let f = ECP4::frob_constants();
+    let x = BIG::new_ints(&rom::CURVE_BNX);
+    let mut W=ECP4::new(); W.copy(P); W.frob(&f,1);
+    let mut T=P.mul(&x);
+    if ecp::SIGN_OF_X == ecp::NEGATIVEX {
+        T.neg();
+    }
+    if !W.equals(&T) {
         return false;
     }
+/*
+    let W=P.mul(&q); 
+    if !W.is_infinity() {
+        return false;
+    } */
     true
 }
 
-/* test GT group membership */
-/* First check that m!=1, conj(m)*m==1, and m.m^{p^8}=m^{p^4} */
-pub fn gtmember(m: &FP24) -> bool {
+/* Check that m is in cyclotomic sub-group */
+/* Check that m!=1, conj(m)*m==1, and m.m^{p^8}=m^{p^4} */
+pub fn gtcyclotomic(m: &FP24) -> bool {
     if m.isunity() {
         return false;
     }
@@ -837,12 +880,32 @@ pub fn gtmember(m: &FP24) -> bool {
     if !w.equals(&r) {
         return false;
     }
-    let q = BIG::new_ints(&rom::CURVE_ORDER);
-    w.copy(&m);
-    r.copy(&gtpow(&w,&q));
-    if !r.isunity() {
+    return true;
+}
+
+/* test for full GT group membership */
+pub fn gtmember(m: &FP24) -> bool {
+    if !gtcyclotomic(m) {
         return false;
     }
+    let f = FP2::new_bigs(&BIG::new_ints(&rom::FRA), &BIG::new_ints(&rom::FRB));    
+    let x = BIG::new_ints(&rom::CURVE_BNX);
+    let mut r=FP24::new_copy(m); r.frob(&f,1);
+    let mut t=m.pow(&x);
+
+    if ecp::SIGN_OF_X == ecp::NEGATIVEX {
+        t.conj();
+    }
+    
+    if !r.equals(&t) {
+        return false;
+    }
+/*
+    let q = BIG::new_ints(&rom::CURVE_ORDER);
+    let r = m.pow(&q);
+    if !r.isunity() {
+        return false;
+    } */
     true
 }
 
