@@ -45,7 +45,7 @@ impl std::fmt::Debug for FP48 {
     fn fmt(&self, formatter: &mut std::fmt::Formatter) -> std::fmt::Result {
         write!(formatter, "{}", self.tostring())
     }
-}
+}    
 
 #[cfg(feature = "std")]
 impl std::fmt::Display for FP48 {
@@ -137,7 +137,7 @@ impl FP48 {
         self.b.cmove(&g.b, d);
         self.c.cmove(&g.c, d);
         let mut u = d as usize;
-        u = !(u - 1);
+        u = !(u.wrapping_sub(1));
         self.stype ^= (self.stype ^ g.stype) & u;
     }
 
@@ -884,44 +884,44 @@ impl FP48 {
 
     /* convert from byte array to FP48 */
     pub fn frombytes(w: &[u8]) -> FP48 {
-        const MB: usize = 16 * (big::MODBYTES as usize);
+        const MB:usize = 16*(big::MODBYTES as usize);
         let mut t: [u8; MB] = [0; MB];
-        for i in 0..MB {
-            t[i] = w[i];
-        }
-        let c = FP16::frombytes(&t);
-        for i in 0..MB {
-            t[i] = w[i + MB];
-        }
-        let b = FP16::frombytes(&t);
-        for i in 0..MB {
-            t[i] = w[i + 2 * MB];
-        }
-        let a = FP16::frombytes(&t);
-        FP48::new_fp16s(&a, &b, &c)
+	    for i in 0..MB {
+		    t[i]=w[i];
+	    }
+        let c=FP16::frombytes(&t);
+	    for i in 0..MB {
+		    t[i]=w[i+MB];
+	    }
+        let b=FP16::frombytes(&t);
+	    for i in 0..MB {
+		    t[i]=w[i+2*MB];
+	    }
+        let a=FP16::frombytes(&t);
+	FP48::new_fp16s(&a,&b,&c)
     }
 
     /* convert this to byte array */
     pub fn tobytes(&mut self, w: &mut [u8]) {
-        const MB: usize = 16 * (big::MODBYTES as usize);
+        const MB:usize = 16*(big::MODBYTES as usize);
         let mut t: [u8; MB] = [0; MB];
 
         self.c.tobytes(&mut t);
-        for i in 0..MB {
-            w[i] = t[i];
-        }
+	    for i in 0..MB { 
+		    w[i]=t[i];
+	    }
         self.b.tobytes(&mut t);
-        for i in 0..MB {
-            w[i + MB] = t[i];
-        }
+	    for i in 0..MB {
+		    w[i+MB]=t[i];
+	    }
         self.a.tobytes(&mut t);
-        for i in 0..MB {
-            w[i + 2 * MB] = t[i];
-        }
+	    for i in 0..MB {
+		    w[i+2*MB]=t[i];
+	    }
     }
 
     /* output to hex string */
-    #[cfg(feature = "std")]
+#[cfg(feature = "std")]
     pub fn tostring(&self) -> String {
         format!(
             "[{},{},{}]",
@@ -931,9 +931,9 @@ impl FP48 {
         )
     }
 
-    /* Note this is simple square and multiply, so not side-channel safe */
-    /* But fast for final exponentiation where exponent is not a secret */
-    /* return this^e */
+/* Note this is simple square and multiply, so not side-channel safe */
+/* But fast for final exponentiation where exponent is not a secret */
+/* return this^e */
     pub fn pow(&self, e: &BIG) -> FP48 {
         let mut r = FP48::new_copy(self);
         r.norm();
@@ -978,43 +978,43 @@ impl FP48 {
         }
         self.copy(&r[0]);
     }
-    /*
-        pub fn compow(&mut self, e: &BIG, r: &BIG) -> FP16 {
-            let f = FP2::new_bigs(&BIG::new_ints(&rom::FRA), &BIG::new_ints(&rom::FRB));
-            let q = BIG::new_ints(&rom::MODULUS);
+/*
+    pub fn compow(&mut self, e: &BIG, r: &BIG) -> FP16 {
+        let f = FP2::new_bigs(&BIG::new_ints(&rom::FRA), &BIG::new_ints(&rom::FRB));
+        let q = BIG::new_ints(&rom::MODULUS);
 
-            let mut g1 = FP48::new_copy(self);
-            let mut g2 = FP48::new_copy(self);
+        let mut g1 = FP48::new_copy(self);
+        let mut g2 = FP48::new_copy(self);
 
-            let mut m = BIG::new_copy(&q);
-            m.rmod(&r);
+        let mut m = BIG::new_copy(&q);
+        m.rmod(&r);
 
-            let mut a = BIG::new_copy(&e);
-            a.rmod(&mut m);
+        let mut a = BIG::new_copy(&e);
+        a.rmod(&mut m);
 
-            let mut b = BIG::new_copy(&e);
-            b.div(&mut m);
+        let mut b = BIG::new_copy(&e);
+        b.div(&mut m);
 
-            let mut c = g1.trace();
-
-            if b.iszilch() {
-                c = c.xtr_pow(&mut a);
-                return c;
-            }
-
-            g2.frob(&f, 1);
-            let cp = g2.trace();
-            g1.conj();
-            g2.mul(&g1);
-            let cpm1 = g2.trace();
-            g2.mul(&g1);
-            let cpm2 = g2.trace();
-
-            c = c.xtr_pow2(&cp, &cpm1, &cpm2, &mut a, &mut b);
+        let mut c = g1.trace();
 
+        if b.iszilch() {
+            c = c.xtr_pow(&mut a);
             return c;
         }
-    */
+
+        g2.frob(&f, 1);
+        let cp = g2.trace();
+        g1.conj();
+        g2.mul(&g1);
+        let cpm1 = g2.trace();
+        g2.mul(&g1);
+        let cpm2 = g2.trace();
+
+        c = c.xtr_pow2(&cp, &cpm1, &cpm2, &mut a, &mut b);
+
+        return c;
+    }
+*/
     /* p=q0^u0.q1^u1.q2^u2.q3^u3... */
     // Bos & Costello https://eprint.iacr.org/2013/458.pdf
     // Faz-Hernandez & Longa & Sanchez  https://eprint.iacr.org/2013/158.pdf
